from .renderers import ChoiceFieldRenderer, FieldRenderer


class MailRenderersLibrary:
    def __init__(self):
        self.storage = {}

    def register(self, model_field):
        def dec(renderer):
            self.storage[model_field] = renderer
            return renderer

        return dec

    def get_renderer(self, field):
        if field.choices:
            return ChoiceFieldRenderer(field)
        try:
            return self.storage[type(field)](field)
        except KeyError:
            return FieldRenderer(field)


renderers_library = MailRenderersLibrary()



from typing import TYPE_CHECKING, Union, List

from django.core.mail import EmailMessage
from django.forms import ALL_FIELDS

from .library import renderers_library

if TYPE_CHECKING:
    from django.db.models.base import Model


def send_model_mail(
    obj: "Model",
    to: List[str],
    subject: str,
    fields: Union[str, List[str]] = ALL_FIELDS,
    template_name: str = "model_mail/email.html",
):
    body = subject + "\n"
    for field_name in fields:
        field = obj._meta.get_field(field_name)
        value = getattr(obj, field_name)
        body += renderers_library.get_renderer(field).render(value) + "\n"
    message = EmailMessage(
        subject,
        body,
        to=to,
    )
    message.send()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['model_mail']

package_data = \
{'': ['*'], 'model_mail': ['templates/model_mail/*']}

install_requires = \
['Django>=3.0.0,<4.0.0']

setup_kwargs = {
    'name': 'django-model-mail',
    'version': '0.2.2',
    'description': "Django package that helps to create emails from models (e.g. for admins' notifications)",
    'long_description': "=================\nDjango Model Mail\n=================\n\n\n.. image:: https://img.shields.io/pypi/v/django-model-mail.svg\n        :target: https://pypi.python.org/pypi/django-model-mail\n\n.. image:: https://img.shields.io/travis/la1t/django-model-mail.svg\n        :target: https://travis-ci.org/la1t/django-model-mail\n\n\nDjango package that helps to create emails from models (e.g. for admins' notifications)\n\n\n* Free software: MIT license\n\n\nFeatures\n--------\n\n* TODO\n\n\nDeploying\n---------\n\nA reminder for the maintainers on how to deploy.\nMake sure all your changes are committed (including an entry in HISTORY.rst).\nThen run::\n\n$ poetry version patch # possible: major / minor / patch\n$ git push\n$ git push --tags\n\nTravis will then deploy to PyPI if tests pass.\n",
    'author': 'Anatoly Gusev',
    'author_email': 'gusev.tolia@yandex.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/la1t/django-model-mail',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from django.utils.hashable import make_hashable
from django.utils.encoding import force_str


def get_choice_display(field, value):
    choices_dict = dict(make_hashable(field.flatchoices))
    # force_str() to coerce lazy strings.
    return force_str(choices_dict.get(make_hashable(value), value), strings_only=True)



from .utils import get_choice_display


class FieldRenderer:
    def __init__(self, field):
        self.field = field

    def render(self, value):
        return f"{self.render_label()}: {self.render_value(value)}"

    def render_label(self):
        return self.field.verbose_name

    def render_value(self, value):
        return value


class ChoiceFieldRenderer(FieldRenderer):
    def render_value(self, value):
        return get_choice_display(self.field, value)
